@@ -54,6 +54,10 @@ class MD_Metadata(object):
         val = md.find(util.nspath_eval('gmd:dateStamp/gco:Date', namespaces))
         self.datestamp = util.testXMLValue(val)
 
+        if not self.datestamp:
+            val = md.find(util.nspath_eval('gmd:dateStamp/gco:DateTime', namespaces))
+            self.datestamp = util.testXMLValue(val)
+
         self.charset = _testCodeListValue(md.find(util.nspath_eval('gmd:characterSet/gmd:MD_CharacterSetCode', namespaces)))
   
         self.hierarchy = _testCodeListValue(md.find(util.nspath_eval('gmd:hierarchyLevel/gmd:MD_ScopeCode', namespaces)))
@@ -117,10 +121,7 @@ class CI_Date(object):
                 self.date = None
 
         val = md.find(util.nspath_eval('gmd:dateType/gmd:CI_DateTypeCode', namespaces))
-        if val is not None:
-            self.type = util.testXMLValue(val)
-        else:
-            self.type = util.testXMLValue(md.attrib.get('codeListValue'), True)
+        self.type = _testCodeListValue(val)
 
 class CI_ResponsibleParty(object):
     """ process CI_ResponsibleParty """
@@ -194,13 +195,13 @@ class MD_DataIdentification(object):
         
         self.accessconstraints = []
         for i in md.findall(util.nspath_eval('gmd:resourceConstraints/gmd:MD_LegalConstraints/gmd:accessConstraints/gmd:MD_RestrictionCode', namespaces)):
-            val = util.testXMLValue(i)
+            val = _testCodeListValue(i)
             if val is not None:
                 self.accessconstraints.append(val)
         
         self.classification = []
         for i in md.findall(util.nspath_eval('gmd:resourceConstraints/gmd:MD_LegalConstraints/gmd:accessConstraints/gmd:MD_ClassificationCode', namespaces)):
-            val = util.testXMLValue(i)
+            val = _testCodeListValue(i)
             if val is not None:
                 self.classification.append(val)
         
@@ -215,6 +216,12 @@ class MD_DataIdentification(object):
             val = util.testXMLValue(i)
             if val is not None:
                 self.securityconstraints.append(val)
+
+        self.useconstraints = []
+        for i in md.findall(util.nspath_eval('gmd:resourceConstraints/gmd:MD_LegalConstraints/gmd:useConstraints/gmd:MD_RestrictionCode', namespaces)):
+            val = _testCodeListValue(i)
+            if val is not None:
+                self.useconstraints.append(val)
         
         self.denominators = []
         for i in md.findall(util.nspath_eval('gmd:spatialResolution/gmd:MD_Resolution/gmd:equivalentScale/gmd:MD_RepresentativeFraction/gmd:denominator/gco:Integer', namespaces)):
@@ -232,7 +239,7 @@ class MD_DataIdentification(object):
         
         self.resourcelanguage = []
         for i in md.findall(util.nspath_eval('gmd:language/gmd:LanguageCode', namespaces)):
-            val = util.testXMLValue(i)
+            val = _testCodeListValue(i)
             if val is not None:
                 self.resourcelanguage.append(val)
 
@@ -285,10 +292,13 @@ class MD_DataIdentification(object):
 
             for k in i.findall(util.nspath_eval('gmd:MD_Keywords/gmd:keyword', namespaces)):
                 val = k.find(util.nspath_eval('gco:CharacterString', namespaces))
-                mdkw['keywords'].append(util.testXMLValue(val))
+                if val is not None:
+                    val2 = util.testXMLValue(val) 
+                    if val2 is not None:
+                        mdkw['keywords'].append(val2)
 
             self.keywords.append(mdkw)
-        
+
         self.topiccategory = []
         for i in md.findall(util.nspath_eval('gmd:topicCategory/gmd:MD_TopicCategoryCode', namespaces)):
             val = util.testXMLValue(i)
@@ -298,12 +308,16 @@ class MD_DataIdentification(object):
         val = md.find(util.nspath_eval('gmd:supplementalInformation/gco:CharacterString', namespaces))
         self.supplementalinformation = util.testXMLValue(val)
         
-        val = md.find(util.nspath_eval('gmd:extent/gmd:EX_Extent/gmd:geographicElement', namespaces))
-
-        if val is not None:
-            self.bbox = EX_Extent(val)
-        else:
-            self.bbox = None
+        # There may be multiple geographicElement, create an extent
+        # from the one containing either an EX_GeographicBoundingBox or EX_BoundingPolygon.
+        # The schema also specifies an EX_GeographicDescription. This is not implemented yet.
+        val = None
+        for e in md.findall(util.nspath_eval('gmd:extent/gmd:EX_Extent/gmd:geographicElement', namespaces)):
+            if e.find(util.nspath_eval('gmd:EX_GeographicBoundingBox', namespaces)) is not None or e.find(util.nspath_eval('gmd:EX_BoundingPolygon', namespaces)) is not None:
+                val = e
+                break
+        self.extent = EX_Extent(val)
+        self.bbox = self.extent.boundingBox  # for backwards compatibility
         
         val = md.find(util.nspath_eval('gmd:extent/gmd:EX_Extent/gmd:temporalElement/gmd:EX_TemporalExtent/gmd:extent/gml:TimePeriod/gml:beginPosition', namespaces))
         self.temporalextent_start = util.testXMLValue(val)
@@ -344,7 +358,7 @@ class DQ_DataQuality(object):
         
         self.conformancedatetype = []
         for i in md.findall(util.nspath_eval('gmd:report/gmd:DQ_DomainConsistency/gmd:result/gmd:DQ_ConformanceResult/gmd:specification/gmd:CI_Citation/gmd:date/gmd:CI_Date/gmd:dateType/gmd:CI_DateTypeCode', namespaces)):
-            val = util.testXMLValue(i)
+            val = _testCodeListValue(i)
             if val is not None:
                 self.conformancedatetype.append(val)
         
@@ -431,18 +445,63 @@ class CI_OnlineResource(object):
 
         self.function = _testCodeListValue(md.find(util.nspath_eval('gmd:function/gmd:CI_OnLineFunctionCode', namespaces)))
 
+
+class EX_GeographicBoundingBox(object):
+    def __init__(self, md):
+        val = md.find(util.nspath_eval('gmd:westBoundLongitude/gco:Decimal', namespaces))
+        self.minx = util.testXMLValue(val)
+        val = md.find(util.nspath_eval('gmd:eastBoundLongitude/gco:Decimal', namespaces))
+        self.maxx = util.testXMLValue(val)
+        val = md.find(util.nspath_eval('gmd:southBoundLatitude/gco:Decimal', namespaces))
+        self.miny = util.testXMLValue(val)
+        val = md.find(util.nspath_eval('gmd:northBoundLatitude/gco:Decimal', namespaces))
+        self.maxy = util.testXMLValue(val)
+    
+class EX_Polygon(object):
+    def __init__(self, md):
+        linear_ring = md.find(util.nspath_eval('gml32:Polygon/gml32:exterior/gml32:LinearRing', namespaces))
+        if linear_ring is not None:
+            self.exterior_ring = self._coordinates_for_ring(linear_ring)
+                    
+        interior_ring_elements = md.findall(util.nspath_eval('gml32:Polygon/gml32:interior', namespaces))
+        self.interior_rings = []
+        for iring_element in interior_ring_elements:
+            linear_ring = iring_element.find(util.nspath_eval('gml32:LinearRing', namespaces))
+            self.interior_rings.append(self._coordinates_for_ring(linear_ring))
+            
+    def _coordinates_for_ring(self, linear_ring):
+        coordinates = []
+        positions = linear_ring.findall(util.nspath_eval('gml32:pos', namespaces))
+        for pos in positions:
+            tokens = pos.text.split()
+            coords = tuple([float(t) for t in tokens])
+            coordinates.append(coords)
+        return coordinates
+        
+class EX_GeographicBoundingPolygon(object):
+    def __init__(self, md):
+        val = md.find(util.nspath_eval('gmd:extentTypeCode', namespaces))
+        self.is_extent = util.testXMLValue(val)
+        
+        md_polygons = md.findall(util.nspath_eval('gmd:polygon', namespaces))
+        
+        self.polygons = []
+        for val in md_polygons:
+            self.polygons.append(EX_Polygon(val))
+            
 class EX_Extent(object):
     """ process EX_Extent """
     def __init__(self, md):
-        val = md.find(util.nspath_eval('gmd:EX_GeographicBoundingBox/gmd:westBoundLongitude/gco:Decimal', namespaces))
-        self.minx = util.testXMLValue(val)
-        val = md.find(util.nspath_eval('gmd:EX_GeographicBoundingBox/gmd:eastBoundLongitude/gco:Decimal', namespaces))
-        self.maxx = util.testXMLValue(val)
-        val = md.find(util.nspath_eval('gmd:EX_GeographicBoundingBox/gmd:southBoundLatitude/gco:Decimal', namespaces))
-        self.miny = util.testXMLValue(val)
-        val = md.find(util.nspath_eval('gmd:EX_GeographicBoundingBox/gmd:northBoundLatitude/gco:Decimal', namespaces))
-        self.maxy = util.testXMLValue(val)
-
+        self.boundingBox = None
+        self.boundingPolygon = None
+        bboxElement = md.find(util.nspath_eval('gmd:EX_GeographicBoundingBox', namespaces))
+        if bboxElement is not None:
+            self.boundingBox = EX_GeographicBoundingBox(bboxElement)
+        
+        polygonElement = md.find(util.nspath_eval('gmd:EX_BoundingPolygon', namespaces))
+        if polygonElement is not None:
+            self.boundingPolygon = EX_GeographicBoundingPolygon(polygonElement)
+ 
         val = md.find(util.nspath_eval('gmd:EX_GeographicDescription/gmd:geographicIdentifier/gmd:MD_Identifier/gmd:code/gco:CharacterString', namespaces))
         self.description_code = util.testXMLValue(val)
 
@@ -453,8 +512,13 @@ class MD_ReferenceSystem(object):
         self.code = util.testXMLValue(val)
 
 def _testCodeListValue(elpath):
-    if elpath is not None:
-        return util.testXMLValue(elpath.attrib.get('codeListValue'), True)
+    """ get gco:CodeListValue_Type attribute, else get text content """
+    if elpath is not None:  # try to get @codeListValue
+        val = util.testXMLValue(elpath.attrib.get('codeListValue'), True)
+        if val is not None:
+            return val
+        else:  # see if there is element text
+            return util.testXMLValue(elpath)
     else:
         return None
 
@@ -475,21 +539,21 @@ class CodelistCatalogue(object):
         self.dictionaries = {}
 
         for i in ct.findall(util.nspath_eval('gmx:codelistItem/gmx:CodeListDictionary', namespaces)):
-            id = i.attrib.get(util.nspath_eval('gml:id', namespaces))
+            id = i.attrib.get(util.nspath_eval('gml32:id', namespaces))
             self.dictionaries[id] = {}
-            val = i.find(util.nspath_eval('gml:description', namespaces))
+            val = i.find(util.nspath_eval('gml32:description', namespaces))
             self.dictionaries[id]['description'] = util.testXMLValue(val)
-            val = i.find(util.nspath_eval('gml:identifier', namespaces))
+            val = i.find(util.nspath_eval('gml32:identifier', namespaces))
             self.dictionaries[id]['identifier'] = util.testXMLValue(val)
             self.dictionaries[id]['entries'] = {}
 
             for j in i.findall(util.nspath_eval('gmx:codeEntry', namespaces)):
-                id2 = j.find(util.nspath_eval('gmx:CodeDefinition', namespaces)).attrib.get(util.nspath_eval('gml:id', namespaces))
+                id2 = j.find(util.nspath_eval('gmx:CodeDefinition', namespaces)).attrib.get(util.nspath_eval('gml32:id', namespaces))
                 self.dictionaries[id]['entries'][id2] = {}
-                val = j.find(util.nspath_eval('gmx:CodeDefinition/gml:description', namespaces))
+                val = j.find(util.nspath_eval('gmx:CodeDefinition/gml32:description', namespaces))
                 self.dictionaries[id]['entries'][id2]['description'] = util.testXMLValue(val)
 
-                val = j.find(util.nspath_eval('gmx:CodeDefinition/gml:identifier', namespaces))
+                val = j.find(util.nspath_eval('gmx:CodeDefinition/gml32:identifier', namespaces))
                 self.dictionaries[id]['entries'][id2]['identifier'] = util.testXMLValue(val)
 
                 val = j.find(util.nspath_eval('gmx:CodeDefinition', namespaces)).attrib.get('codeSpace')
@@ -506,3 +570,4 @@ class CodelistCatalogue(object):
             return ids
         else:
             return None
+
